program CsvFiles;
uses SysUtils, StrUtils, Classes, db, memds;

procedure SaveToCsv(DataSet:TDataSet);
var
  myFileName:string;
  myTextFile: TextFile;
  i: integer;
  s,stype: string;
begin
  myFileName:=DataSet.Name+'.txt';
  //create a new file
  AssignFile(myTextFile, myFileName);
  Rewrite(myTextFile);
  s := ''; //initialize empty string
  try
    //write field names (as column headers)
    for i := 0 to DataSet.Fields.Count - 1 do
      begin
         case Dataset.FieldDefs[i].DataType of
            ftBoolean: stype := 'boolean';
            ftInteger: stype := 'integer';
            ftSmallInt: stype := 'smallint';
            ftFloat: stype := 'float';
            ftString: stype := 'string';
            ftTime: stype := 'time';
            ftDate: stype := 'date';
         end;
         s := s + Format('%s:%s',
                         [DataSet.Fields[i].FieldName, stype]);
         if i < DataSet.Fields.Count - 1 then s := s + ',';
      end;
    Writeln(myTextFile, s);
    DataSet.First;
    //write field values
    while not DataSet.Eof do
      begin
        s := '';
        for i := 0 to DataSet.FieldCount - 1 do
          begin
            //Numerical fields without quotes, string fields with quotes
            if ((DataSet.FieldDefs[i].DataType=ftInteger)
             or (DataSet.FieldDefs[i].DataType=ftFloat)) then
              s := s + Format('%s', [DataSet.Fields[i].AsString])
            else
              s := s + Format('"%s"', [DataSet.Fields[i].AsString]);
            if i < DataSet.Fields.Count - 1 then s := s + ',';
          end;
        Writeln(myTextfile, s);
        DataSet.Next;
      end;
  finally
    CloseFile(myTextFile);
  end;
end;

procedure LoadFromCsv(DataSet:TMemDataSet); overload;
var
  myFileName: String;
  myTextFile: TextFile;
  i: Integer;
  s, stype: String;
  params: TStringList;
  ptype: TFieldType;
begin
  myFileName:=DataSet.Name+'.txt';
  //create a new file
  AssignFile(myTextFile, myFileName);
  Reset(myTextFile);
  ReadLn (myTextFile, s); //first line -- header
  params := TStringList.Create;
  params.Delimiter:=',';
  params.StrictDelimiter:=false; //not sure!
  params.QuoteChar:='"';
  params.DelimitedText:=s;
  for s in params do
  begin
     stype := lowercase (ExtractWord (2, s, [':']));
     if stype = 'boolean' then ptype := ftBoolean
     else if stype = 'integer' then ptype := ftInteger
     else if stype = 'smallint' then ptype := ftSmallInt
     else if stype = 'float' then ptype := ftFloat
     else if stype = 'string' then ptype := ftString
     else if stype = 'time' then ptype := ftTime
     else if stype = 'date' then ptype := ftDate
     else raise (Exception.Create ('LoadFromCsv: unknown type '+stype));
     DataSet.FieldDefs.Add (ExtractWord (1, s, [':']), ptype);
  end;
  DataSet.CreateTable;
  DataSet.Open;
  while not EOF(myTextFile) do
  begin
     ReadLn (myTextFile, s); //first line -- header
     params.DelimitedText:=s;
     DataSet.Append;
     for i:=0 to DataSet.FieldDefs.Count - 1 do
        DataSet.Fields[i].AsString := params[i];
  end;
  CloseFile(myTextFile);
  params.Free;
end;

function LoadFromCsv (DataName: String) : TMemDataset; overload;
begin
   result := TMemDataset.Create(nil);
   result.Name := DataName;
   LoadFromCsv (result)
end;

var
   Data1,Data2 : TMemDataset;
begin
   Data1 := TMemDataset.Create (nil);
   Data1.LoadFromFile ('TestpopResult');
   Data1.Name := 'TestpopResult';
   Data1.Open;
   SaveToCsv (Data1);
   Data1.Close;
   Data2 := LoadFromCsv ('TestpopResult'); //without .txt extension
   Data2.FileName := 'CsvFilesResult';
   Data2.Close;
   Data1.Free; Data2.Free;
end.